/*
 * $Id: Launcher.java 356 2011-07-03 13:54:56Z balder $
 *
 * This file is part of the iText (R) project.
 * Copyright (c) 1998-2011 1T3XT BVBA
 * Authors: Balder Van Camp, et al.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License version 3
 * as published by the Free Software Foundation with the addition of the
 * following permission added to Section 15 as permitted in Section 7(a):
 * FOR ANY PART OF THE COVERED WORK IN WHICH THE COPYRIGHT IS OWNED BY 1T3XT,
 * 1T3XT DISCLAIMS THE WARRANTY OF NON INFRINGEMENT OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA, 02110-1301 USA, or download the license from the following URL:
 * http://itextpdf.com/terms-of-use/
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License,
 * a covered work must retain the producer line in every PDF that is created
 * or manipulated using iText.
 *
 * You can be released from the requirements of the license by purchasing
 * a commercial license. Buying such a license is mandatory as soon as you
 * develop commercial activities involving the iText software without
 * disclosing the source code of your own applications.
 * These activities include: offering paid services to customers as an ASP,
 * serving PDFs on the fly in a web application, shipping iText with a closed
 * source product.
 *
 * For more information, please contact iText Software Corp. at this
 * address: sales@itextpdf.com
 */
package com.itextpdf.maven.itextdoc;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.nio.charset.Charset;
import java.util.Arrays;

import com.itextpdf.text.log.Level;
import com.itextpdf.text.log.Logger;
import com.itextpdf.text.log.LoggerFactory;
import com.itextpdf.text.log.NoOpLogger;
import com.itextpdf.tool.itextdoc.pdf.DocumentationInputStreamReader;
import com.itextpdf.tool.itextdoc.pdf.DocumentationPipeline;
import com.itextpdf.tool.itextdoc.site.FlatSitePipeline;
import com.itextpdf.tool.itextdoc.site.SitePipeline;
import com.itextpdf.tool.xml.Pipeline;
import com.itextpdf.tool.xml.XMLWorker;
import com.itextpdf.tool.xml.XMLWorkerHelper;
import com.itextpdf.tool.xml.html.AbstractTagProcessor;
import com.itextpdf.tool.xml.html.DefaultTagProcessorFactory;
import com.itextpdf.tool.xml.html.DummyTagProcessor;
import com.itextpdf.tool.xml.html.TagProcessorFactory;
import com.itextpdf.tool.xml.html.Tags;
import com.itextpdf.tool.xml.parser.XMLParser;
import com.itextpdf.tool.xml.pipeline.css.CSSResolver;
import com.itextpdf.tool.xml.pipeline.css.CssResolverPipeline;
import com.itextpdf.tool.xml.pipeline.end.PdfWriterPipeline;
import com.itextpdf.tool.xml.pipeline.html.AbstractImageProvider;
import com.itextpdf.tool.xml.pipeline.html.HtmlPipeline;
import com.itextpdf.tool.xml.pipeline.html.HtmlPipelineContext;

/**
 * @author redlab_b http://www.redlab.be/
 *
 */
public class Launcher {

	private static final ThreadLocal<Logger> tl = new ThreadLocal<Logger>() {
		@Override
		protected Logger initialValue() {
			return new NoOpLogger();
		};
	};
	static {
		LoggerFactory.getInstance().setLogger(new ItxtLog());
	}

	/**
	 * @author redlab_b http://www.redlab.be/
	 *
	 */
	protected static final class ItxtLog implements Logger {

		/*
		 * (non-Javadoc)
		 *
		 * @see com.itextpdf.text.log.Logger#debug(java.lang.String)
		 */
		public void debug(final String arg0) {
		}

		/*
		 * (non-Javadoc)
		 *
		 * @see com.itextpdf.text.log.Logger#error(java.lang.String)
		 */
		public void error(final String arg0) {
		}

		/*
		 * (non-Javadoc)
		 *
		 * @see com.itextpdf.text.log.Logger#error(java.lang.String,
		 * java.lang.Exception)
		 */
		public void error(final String arg0, final Exception arg1) {
		}

		/*
		 * (non-Javadoc)
		 *
		 * @see com.itextpdf.text.log.Logger#getLogger(java.lang.Class)
		 */
		public Logger getLogger(final Class<?> arg0) {
			return tl.get().getLogger(arg0);
		}

		/*
		 * (non-Javadoc)
		 *
		 * @see com.itextpdf.text.log.Logger#getLogger(java.lang.String)
		 */
		public Logger getLogger(final String arg0) {
			return tl.get().getLogger(arg0);
		}

		/*
		 * (non-Javadoc)
		 *
		 * @see com.itextpdf.text.log.Logger#info(java.lang.String)
		 */
		public void info(final String arg0) {
		}

		/*
		 * (non-Javadoc)
		 *
		 * @see
		 * com.itextpdf.text.log.Logger#isLogging(com.itextpdf.text.log.Level)
		 */
		public boolean isLogging(final Level arg0) {
			return false;
		}

		/*
		 * (non-Javadoc)
		 *
		 * @see com.itextpdf.text.log.Logger#trace(java.lang.String)
		 */
		public void trace(final String arg0) {
		}

		/*
		 * (non-Javadoc)
		 *
		 * @see com.itextpdf.text.log.Logger#warn(java.lang.String)
		 */
		public void warn(final String arg0) {
		}

	}

	/**
	 * @author redlab_b http://www.redlab.be/
	 *
	 */
	protected final class MojoLogger implements Logger {
		/**
		 *
		 */
		private final ItextDocMojo mojo;

		/**
		 * @param mojo the MavenMojo
		 */
		protected MojoLogger(final ItextDocMojo mojo) {
			this.mojo = mojo;
		}

		public void warn(final String message) {
			mojo.getLog().warn(message);
		}

		public void trace(final String message) {
		}

		public boolean isLogging(final Level level) {
			if (Level.DEBUG == level) {
				return mojo.getLog().isDebugEnabled();
			}
			if (Level.ERROR == level) {
				return mojo.getLog().isErrorEnabled();
			}
			if (Level.INFO == level) {
				return mojo.getLog().isInfoEnabled();
			}
			if (Level.WARN == level) {
				return mojo.getLog().isWarnEnabled();
			}
			return false;
		}

		public void info(final String message) {
			mojo.getLog().info(message);
		}

		public Logger getLogger(final String name) {
			return this;
		}

		public Logger getLogger(final Class<?> klass) {
			return this;
		}

		public void error(final String message, final Exception e) {
			mojo.getLog().error(message, e);
		}

		public void error(final String message) {
			mojo.getLog().error(message);
		}

		public void debug(final String message) {
			mojo.getLog().debug(message);
		}
	}

	private DefaultTagProcessorFactory tagFactory;
	private HtmlPipelineContext hpc;
	private final boolean site;
	private final boolean flatsite;
	private final String encoding;

	/**
	 * @param mojo the MavenMojo
	 */
	public Launcher(final ItextDocMojo mojo) {
		tl.set(new MojoLogger(mojo));
		//
		this.site = mojo.getSite();
		this.flatsite = mojo.getFlatSite();
		this.encoding = mojo.getEncoding();
	}

	/**
	 *
	 * @param docRoot the root where the doc.xml resides.
	 * @param result the directory where the results should end up.
	 * @throws IOException if something could not be read or written.
	 */
	public void parse(final String docRoot, final String result) throws IOException {
		final File docRootFile = new File(docRoot);
		final File resultdir = new File(result);
		parse(docRootFile, resultdir);
	}

	private synchronized HtmlPipelineContext createContext(final File srcDir) {
		if (null == hpc) {
			hpc = new HtmlPipelineContext();
			TagProcessorFactory tagFactory = getTagFactory();
			hpc.autoBookmark(false).setTagFactory(tagFactory).setImageProvider(new AbstractImageProvider() {

				public String getImageRootPath() {
					return srcDir.getPath() + File.separator;
				}
			});
			hpc.setRootTags(Arrays.asList(new String[] { "body", "part" }));
			hpc.charSet(Charset.forName(encoding));
		}
		return hpc;
	}

	private synchronized TagProcessorFactory getTagFactory() {
		if (null == tagFactory) {
			tagFactory = (DefaultTagProcessorFactory) Tags.getHtmlTagProcessorFactory();
			DummyTagProcessor dummy = new DummyTagProcessor();
			tagFactory.addProcessor(dummy, "html", "head", "script", "title");
			tagFactory.addProcessor(new AbstractTagProcessor() {
				// we only want the defaults
			}, "part");
		}
		return tagFactory;
	}

	/**
	 * @param in the file to parse
	 * @param f the file to write to.
	 * @throws IOException if something could not be read or written.
	 */
	public void parse(final File in, final File f) throws IOException {
		f.mkdirs();
		final DocumentationInputStreamReader dis = new DocumentationInputStreamReader(new FileInputStream(
				new File(in, "doc.xml")), encoding);
		HtmlPipelineContext hpc = createContext(in);
		final CSSResolver css = XMLWorkerHelper.getInstance().getDefaultCssResolver(true);
		Pipeline<?> hp = new DocumentationPipeline(new com.itextpdf.tool.itextdoc.pdf.Config() {

			public File docRoot() {
				return in;
			}

			public DocumentationInputStreamReader stream() {
				return dis;
			}

			public File resultdir() {
				return f;
			}

			public CSSResolver css() {
				return css;
			}

		}, new HtmlPipeline(hpc,
				new PdfWriterPipeline()));
		if (site) {
			hp = new SitePipeline(in, f, hp);
		}
		if (flatsite) {
			hp = new FlatSitePipeline(in, f, hp);
		}
		hp = new CssResolverPipeline(css, hp);
		XMLWorker worker = new XMLWorker(hp, true );
		XMLParser parser = new XMLParser(true, worker, Charset.forName(encoding));
		parser.parse(dis);
	}

}
