package com.itextpdf.tool.itextdoc.pdf;

import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.io.UnsupportedEncodingException;
import java.util.LinkedList;

/**
 * @author redlab_b http://www.redlab.be/
 *
 */
public class DocumentationInputStreamReader extends Reader {


	private final LinkedList<Reader> stack;
	private final String charsetName;

	/**
	 * @param in the InputStream
	 * @param charsetName the name of the charset
	 * @throws UnsupportedEncodingException if the charset is not know to the system.
	 */
	public DocumentationInputStreamReader(final InputStream in, final String charsetName) throws UnsupportedEncodingException {
		this.charsetName = charsetName;
		stack = new LinkedList<Reader>();
		stack.add(new InputStreamReader(in, charsetName));
	}


	@Override
	public int read(final char[] cbuf, final int off, final int len) throws IOException {
		int read = -1;
		while (null != stack.peekLast() && (-1 == (read = stack.peekLast().read(cbuf, off, len)))) {
			Reader poll = stack.removeLast();
			poll.close();
		}
		return read;
	}

	/**
	 * Switches the source stream with the given InputStream.
	 * @param in the stream to read from
	 * @throws UnsupportedEncodingException if the charset is not know to the system.
	 */
	public synchronized void switchStream(final InputStream in) throws UnsupportedEncodingException {
		stack.offerLast(new InputStreamReader(in, charsetName));
	}

	@Override
	public void close() throws IOException {
		for (Reader i : stack) {
			i.close();
		}
	}

}
