/*
 * $Id: FlatSitePipeline.java 356 2011-07-03 13:54:56Z balder $
 *
 * This file is part of the iText (R) project.
 * Copyright (c) 1998-2011 1T3XT BVBA
 * Authors: Balder Van Camp, et al.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License version 3
 * as published by the Free Software Foundation with the addition of the
 * following permission added to Section 15 as permitted in Section 7(a):
 * FOR ANY PART OF THE COVERED WORK IN WHICH THE COPYRIGHT IS OWNED BY 1T3XT,
 * 1T3XT DISCLAIMS THE WARRANTY OF NON INFRINGEMENT OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA, 02110-1301 USA, or download the license from the following URL:
 * http://itextpdf.com/terms-of-use/
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License,
 * a covered work must retain the producer line in every PDF that is created
 * or manipulated using iText.
 *
 * You can be released from the requirements of the license by purchasing
 * a commercial license. Buying such a license is mandatory as soon as you
 * develop commercial activities involving the iText software without
 * disclosing the source code of your own applications.
 * These activities include: offering paid services to customers as an ASP,
 * serving PDFs on the fly in a web application, shipping iText with a closed
 * source product.
 *
 * For more information, please contact iText Software Corp. at this
 * address: sales@itextpdf.com
 */
package com.itextpdf.tool.itextdoc.site;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.net.URL;
import java.text.SimpleDateFormat;
import java.util.Date;

import org.apache.commons.io.FileUtils;
import org.apache.commons.io.output.ByteArrayOutputStream;

import com.itextpdf.text.log.Logger;
import com.itextpdf.text.log.LoggerFactory;
import com.itextpdf.tool.xml.Experimental;
import com.itextpdf.tool.xml.Pipeline;
import com.itextpdf.tool.xml.PipelineException;
import com.itextpdf.tool.xml.ProcessObject;
import com.itextpdf.tool.xml.Tag;
import com.itextpdf.tool.xml.WorkerContext;
import com.itextpdf.tool.xml.pipeline.AbstractPipeline;
import com.itextpdf.tool.xml.pipeline.ctx.MapContext;

/**
 * @author redlab_b http://www.redlab.be/
 *
 */
@Experimental
public class FlatSitePipeline extends AbstractPipeline<MapContext> {

	private static final Logger LOG = LoggerFactory.getLogger(FlatSitePipeline.class);
	private final File srcdir;
	private final File outdir;
	private final SiteWriter sitewriter;
	private final AttributeUtil attrUtil;

	/**
	 * @param srcdir the source directory
	 * @param outdir the output directory
	 * @param next the next pipeline in line
	 */
	public FlatSitePipeline(final File srcdir, final File outdir, final Pipeline<?> next) {
		super(next);
		this.sitewriter = new SiteWriter();
		this.attrUtil = new AttributeUtil();
		this.srcdir = srcdir;
		this.outdir = outdir;
	}

	/**
	 * Create tree.js:
	 *
	 * <pre>
	 *  var tree = {
	 *  	menu : [ {name: "info", url : "info.html", id : ""}, {name: "links", url: "links.html", id :""}],
	 *  	title: "",
	 * 	map : []
	 * };
	 * </pre>
	 */
	@Override
	public Pipeline<?> open(final WorkerContext ctx, final Tag t, final ProcessObject po) throws PipelineException {
		try {

			if (checkValidPart(t)) {
				createPart(t, ctx);
			} else if ("pdf".equals(t.getName())) {
				createPDF(t, ctx);
			} else if ("body".equalsIgnoreCase(t.getName())) {
				createBody(ctx);
			} else if ("css".equalsIgnoreCase(t.getName())
					&& (attrUtil.contains("flat", t.getAttributes().get("rel")))) {
				createCSS(t, ctx);
			}

		} catch (IOException e) {
			throw new PipelineException("Unable to copy default site", e);
		}
		return super.open(ctx, t, po);
	}

	/**
	 *
	 * @param t
	 * @param ctx
	 * @throws PipelineException
	 * @throws IOException
	 */
	private void createCSS(final Tag t, final WorkerContext ctx) throws PipelineException, IOException {
		MapContext localContext = getLocalContext(ctx);
		localContext.put("gotcss", "true");
		ByteArrayOutputStream baos = (ByteArrayOutputStream) localContext.get("buffer");
		if ("true".equalsIgnoreCase(t.getAttributes().get("default"))) {
			URL css = FlatSitePipeline.class.getResource("/site/flatstyle.css");
			FileUtils.copyURLToFile(css, new File(outdir, "flatstyle.css"));
			sitewriter.writeStyleLink(baos, "flatstyle.css");
		}
		FileUtils.copyFile(new File(srcdir, t.getAttributes().get("file")), new File(outdir, t.getAttributes()
				.get("file")));
		sitewriter.writeStyleLink(baos, t.getAttributes().get("file"));
	}

	/**
	 * @param ctx
	 * @throws PipelineException
	 * @throws IOException
	 */
	private void createBody(final WorkerContext ctx) throws PipelineException, IOException {
		ByteArrayOutputStream baos = (ByteArrayOutputStream) getLocalContext(ctx).get("buffer");
		if (!("true".equals(getLocalContext(ctx).get("gotcss")))) {
			URL css = FlatSitePipeline.class.getResource("/site/flatstyle.css");
			FileUtils.copyURLToFile(css, new File(outdir, "flatstyle.css"));
			sitewriter.writeStyleLink(baos, "flatstyle.css");
		}
		sitewriter.writeHeadCloseBodyOpen(baos);
	}
	/**
	 *
	 * @param t
	 * @param ctx
	 * @throws PipelineException
	 * @throws IOException
	 */
	private void createPDF(final Tag t, final WorkerContext ctx) throws PipelineException, IOException {
		String title = t.getAttributes().get("title");
		if (null == title) {
			title = "Generated Site by itextdoc-plugin";
		}
		final MapContext mapContext = getLocalContext(ctx);
		mapContext.put("title", title);
		ByteArrayOutputStream baos = (ByteArrayOutputStream) mapContext.get("buffer");
		sitewriter.writeHeadOpen(baos, System.getProperty("file.encoding")).writeTitle(baos, title);
		LOG.debug("Creating flat site for " + title);
	}
	/**
	 *
	 * @param t
	 * @param ctx
	 * @throws PipelineException
	 * @throws IOException
	 */
	private void createPart(final Tag t, final WorkerContext ctx) throws PipelineException, IOException {
		final String title = t.getAttributes().get("title");
		final MapContext mapContext = getLocalContext(ctx);
		final ByteArrayOutputStream menu = (ByteArrayOutputStream) mapContext.get("menu");
		final ByteArrayOutputStream divsboas = (ByteArrayOutputStream) mapContext.get("divs");
		final String file = t.getAttributes().get("file");

		Integer cnt = (Integer) mapContext.get("counter");
		cnt++;
		mapContext.put("counter", cnt);
		final String id = "itextdoc-menu-" + cnt.toString();
		if (!"false".equalsIgnoreCase(t.getAttributes().get("bookmark"))) {
			Tag local = t;
			final StringBuilder titleb = new StringBuilder();
			while ("part".equals(local.getParent().getName())) {
				titleb.append("&gt; ");
				local = local.getParent();
			}
			titleb.append(title);
			final StringBuilder menubuilder = new StringBuilder();
			menubuilder.append("<li><a href=\"#").append(id).append("\">").append(titleb).append("</a></li>");
			menu.write(menubuilder.toString().getBytes());
		}
		try {
			final StringBuilder divs = new StringBuilder();
			divs.append("<div id=\"").append(id).append("\"><a name=\"").append(id).append("\" ></a>");
			final BufferedReader reader = new BufferedReader(new FileReader(new File(srcdir, file)));
			String line = null;
			while (null != (line = reader.readLine())) {
				divs.append(line).append("\n\r");
			}
			divs.append("<a href=\"#sitetop\">top</a></div>");
			divsboas.write(divs.toString().getBytes());
		} catch (final IOException e) {
			throw new PipelineException("Could not read file", e);
		}
	}

	/**
	 *
	 * @param t the tag to check
	 * @return true if t is part and if there is a 'rel' attribute that it contains 'site'
	 */
	public boolean checkValidPart(final Tag t) {
		return ("part".equalsIgnoreCase(t.getName()) && (!t.getAttributes().containsKey("rel") || attrUtil.contains(
				"site", t.getAttributes().get("rel"))));
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * com.itextpdf.tool.xml.pipeline.AbstractPipeline#close(com.itextpdf.tool
	 * .xml.Tag, com.itextpdf.tool.xml.ProcessObject)
	 */
	@Override
	public Pipeline<?> close(final WorkerContext ctx, final Tag t, final ProcessObject po) throws PipelineException {
		if ("pdf".equalsIgnoreCase(t.getName())) {
			FileOutputStream out = null;
			final MapContext mapContext = getLocalContext(ctx);
			String title = t.getAttributes().get("title");
			final ByteArrayOutputStream baos = (ByteArrayOutputStream) mapContext.get("buffer");
			StringBuilder b = new StringBuilder();
			// write menu
			b.append("<div id=\"header\">").append(title).append("</div><div id=\"menu\"><ul>")
					.append(mapContext.get("menu")).append("</ul></div>");
			// write divs
			b.append("<div id=\"content\"><a name=\"sitetop\"></a>").append(mapContext.get("divs")).append("</div>");

			try {
				SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd HH:mm");
				b.append("<div id=\"footer\"><a href=\"").append(t.getAttributes().get("filename"))
						.append("\">Download PDF</a> - <a href=\"index.html\"> view ajax site</a> generated on : ")
						.append(sdf.format(new Date())).append("</div>");
				baos.write(b.toString().getBytes());
				sitewriter.writeDivContent(baos, FlatSitePipeline.class.getResourceAsStream("/site/notice.html"),
						"notice");
				LOG.info("Writing flatsite.html");
				out = new FileOutputStream(new File(outdir, "flatsite.html"));
				baos.writeTo(out);
			} catch (FileNotFoundException e) {
				throw new PipelineException("Unable to create flatsite.html", e);
			} catch (IOException e) {
				throw new PipelineException("Unable to write tree.js", e);
			} finally {
				try {
					baos.close();
				} catch (IOException e) {
					LOG.error("itextdoc sitebuilder Error Closing Stream", e);
				}
				if (null != out) {
					try {
						out.close();
					} catch (IOException e) {
						LOG.error("itextdoc sitebuilder Error Closing Stream", e);
					}
				}
			}
		}
		return super.close(ctx, t, po);
	}

	/* (non-Javadoc)
	 * @see com.itextpdf.tool.xml.pipeline.AbstractPipeline#init(com.itextpdf.tool.xml.WorkerContext)
	 */
	@Override
	public Pipeline<?> init(final WorkerContext context) throws PipelineException {
		MapContext mapContext = new MapContext();
		mapContext.put("menu", new ByteArrayOutputStream());
		mapContext.put("divs", new ByteArrayOutputStream());
		mapContext.put("buffer", new ByteArrayOutputStream());
		mapContext.put("gotcss", "false");
		mapContext.put("counter", Integer.valueOf(0));
		context.put(getContextKey(), mapContext);
		return super.init(context);
	}
}
